"""Facet definitions."""

from invenio_search.engine import dsl
from flask_babelex import lazy_gettext as _

{{ facet_imports|imports }}
{{ vars.facets|imports }}

{% for facet in facets %}
{{ facet.path }} = {{ facet.field }}
{% endfor %}

{{ vars.facets|extra_code }}